#!/usr/bin/python
#identity.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


import os
import sys
import time

from xml.dom import minidom, Node
from xml.dom.minidom import getDOMImplementation

import settings
import dhnio
import nameurl
import misc
import dhncrypto


default_identity_src = """<?xml version="1.0" encoding="ISO-8859-1"?>
<identity>
  <contacts>
  </contacts>
  <sources>
  </sources>
  <certificates>
  </certificates>
  <scrubbers>
  </scrubbers>
  <postage>1
  </postage>
  <date></date>
  <version>100</version>
  <publickey></publickey>
  <signature></signature>
</identity>"""


class identity:
    # Started from page 151 in "Fundations of Python Network Programming"
    # We are passed an XML version of an identity and make an Identity
    sources = []        # list of URLs, first is primary URL and name
    certificates = []   # signatures by identity servers
    publickey = ""      # string in twisted.conch.ssh format
    contacts = []       # list of ways to contact this identity
    scrubbers = []      # list of URLs for people allowed to scrub
    date = ""           # date
    version = "0"       # version string
    signature = ""      # digital signature

    def __init__ (  self,
                    sources = [],
                    certificates = [],
                    publickey = '',
                    contacts = [],
                    scrubbers = [],
                    postage = "1",
                    date = "",
                    version = "0",
                    xmlsrc = default_identity_src,
                    filename = '' ):

        self.sources = sources
        self.certificates = certificates
        self.publickey = publickey
        self.contacts = contacts
        self.scrubbers = scrubbers
        self.postage = postage
        self.date = date
        self.version = version

        if publickey != '':
            self.sign()
        else:
            self.signature=''
            # no point in signing if no public key listed, probably about to unserialize something

        if xmlsrc != '':
            self.unserialize(xmlsrc)

        if filename != '':
            self.unserialize(dhnio.ReadTextFile(filename))

    def clear_data(self):
        self.sources = []      # list of URLs for fetching this identiy, first is primary URL and name - called IDURL
        self.certificates = [] # identity servers each sign the source they are with - hash just (IDURL + publickey)
        self.publickey = ''    # string
        self.contacts = []     # list of ways to contact this identity
        self.scrubbers = []    # list of URLs for people allowed to scrub
        self.date = ''         # date
        self.postage = '1'     # postage price for message delivery if not on correspondents list
        self.version = '0'     # version string
        self.signature = ''    # digital signature

    def isCorrect(self):
        if len(self.contacts) == 0:
            return False
        if len(self.sources) == 0:
            return False
        if self.publickey == '':
            return False
        if self.signature == '':
            return False
        return True

    def getIDURL(self, index = 0):
        result = self.sources[index].strip()
        return result

    def getIDName(self, index = 0):
        protocol, host, port, filename = nameurl.UrlParse(self.getIDURL(index))
        return filename.strip()[0:-4]

    def setIDName(self, name, index = 0):
        protocol, host, port, filename = nameurl.UrlParse(self.getIDURL(index))
        url = nameurl.UrlMake(protocol, host, port, name+'.xml')
        self.sources[index] = url.encode("ascii").strip()

    def clearContacts(self):
        self.contacts = []

    def getContacts(self):
        return self.contacts

    def getContactsNumber(self):
        return len(self.contacts)

    def getContact(self, index=0):
        try:
##            return self.contacts[index].encode("ascii").strip()
            return self.contacts[index]
            # PREPRO don't understand why these were unicode
        except:
            return None

    def setContact(self, contact, index):
##        self.contacts[index] = contact.encode("ascii").strip()
        self.contacts[index] = contact

    def setCertificate(self, certificate):
        self.certificate=certificate
        self.sign()

    def setContactPort(self, index, newport):
        protocol, host, port, filename = nameurl.UrlParse(self.contacts[index])
        url = nameurl.UrlMake(protocol, host, newport, filename)
        self.contacts[index] = url.encode("ascii").strip()

    def getContactHost(self, index):
        protocol, host, port, filename = nameurl.UrlParse(self.contacts[index])
        return host

    def getContactPort(self, index):
        protocol, host, port, filename = nameurl.UrlParse(self.contacts[index])
        return port

    def setContactHost(self, host, index):
        protocol, host_, port, filename = nameurl.UrlParse(self.contacts[index])
        url = nameurl.UrlMake(protocol, host, port, filename)
        self.contacts[index] = url.encode("ascii").strip()

    def getContactParts(self, index):
        return nameurl.UrlParse(self.contacts[index])

    def getProtoParts(self, proto):
        contact = self.getProtoContact(proto)
        if contact is None:
            return None, None, None, None
        return nameurl.UrlParse(contact)

    def getProtoHost(self, proto, default=None):
        protocol, host, port, filename = self.getProtoParts(proto)
        if host is None:
            return default
        return host

    def setContactParts(self, index, protocol, host, port, filename):
        url = nameurl.UrlMake(protocol, host, port, filename)
        self.contacts[index] = url.encode("ascii").strip()

    def getContactIndex(self, proto):
        for i in range(0, len(self.contacts)):
            if self.contacts[i].find(proto+"://") == 0:
                return i
        return -1

    def setProtoContact(self, proto, contact):
        for i in range(0,len(self.contacts)):
            proto_, host, port, filename = nameurl.UrlParse(self.contacts[i])
            if proto_.strip() == proto.strip():
                self.contacts[i] = contact
                return
        self.contacts.append(contact)

    def getProtoContact(self, proto):
        for contact in self.contacts:
            if contact.startswith(proto+"://"):
                return contact
        return None

    def getProtoOrder(self):
        orderL = []
        for c in self.contacts:
            proto,host,port,filename = nameurl.UrlParse(c)
            orderL.append(proto)
        return orderL

    def getContactsByProto(self):
        d = {}
        for i in range(0,len(self.contacts)):
            proto, x, x, x = nameurl.UrlParse(self.contacts[i])
            d[proto] = self.contacts[i]
        return d

    def getContactProto(self, index):
        c = self.getContact(index)
        if c is None:
            return None
        return nameurl.UrlParse(c)[0]

    def getIP(self):
        host = self.getProtoHost('tcp')
        if host is None:
            host = self.getProtoHost('ssh')
        if host is None:
            host = self.getProtoHost('http')
        return host

    def deleteProtoContact(self, proto):
        for contact in self.contacts:
            if contact.find(proto+"://") == 0:
                self.contacts.remove(contact)

    #move given protocol in the bottom of the contacts list
    def pushProtoContact(self, proto):
        i = self.getContactIndex(proto)
        if i < 0:
            return
        contact = self.contacts[i]
        del self.contacts[i]
        self.contacts.append(contact)

    #move given protocol to the top of the contacts list
    def popProtoContact(self, proto):
        i = self.getContactIndex(proto)
        if i < 0:
            return
        contact = self.contacts[i]
        del self.contacts[i]
        self.contacts.insert(0, contact)

    # http://docs.python.org/lib/module-urlparse.html
    # note that certificates and signatures are not part of what is hashed
    def makehash(self):
        sep = "-"
        c = ''
        for i in self.contacts:
            c += i
        s = ''
        for i in self.scrubbers:
            s += i
        sr = ''
        for i in self.sources:
            sr += i
        stufftohash = c + sep + s + sep + sr + sep + self.version + sep + self.postage + sep + self.date.replace(' ', '_')
        #PREPRO thinking of standard that fields have lables and empty fields are left out,
        #including label, so future versions could have same signatures as older which had fewer fields - can just do this for fields after these,
        #so maybe don't need to change anything for now
        hashcode = dhncrypto.Hash(stufftohash)          # don't include certificate - so identity server can just add it
        return hashcode

    def sign(self):
        hashcode = self.makehash()
        self.signature = dhncrypto.Sign(hashcode)
##        if self.Valid():
##            dhnio.Dprint(12, "identity.sign tested after making and it looks good")
##        else:
##            dhnio.Dprint(1, "identity.sign ERROR tested after making sign ")
##            raise Exception("sign fails")

    def Valid(self):
        # PREPRO - should test certificate too
        hashcode = self.makehash()
        result = dhncrypto.VerifySignature(
            self.publickey,
            hashcode,
            str(self.signature))
        return result

    def unserialize(self, xmlsrc):
        try:
            doc = minidom.parseString(xmlsrc)
        except:
            dhnio.DprintException()
            dhnio.Dprint(2, '\n'+xmlsrc[:256]+'\n')
            return
        self.clear_data()
        self.from_xmlobj(doc.documentElement)

    def unserialize_object(self, xmlobject):
        self.clear_data()
        self.from_xmlobj(xmlobject)

    def serialize(self):
        return self.toxml()[0]

    def serialize_object(self):
        return self.toxml()[1]

    def toxml(self):
        impl = getDOMImplementation()
        doc = impl.createDocument(None, 'identity', None)
        root = doc.documentElement
        contacts = doc.createElement('contacts')
        root.appendChild(contacts)
        for contact in self.contacts:
            n = doc.createElement('contact')
            n.appendChild(doc.createTextNode(contact))
            contacts.appendChild(n)

        sources = doc.createElement('sources')
        root.appendChild(sources)
        for source in self.sources:
            n = doc.createElement('source')
            n.appendChild(doc.createTextNode(source))
            sources.appendChild(n)

        certificates = doc.createElement('certificates')
        root.appendChild(certificates)
        for certificate in self.certificates:
            n = doc.createElement('certificate')
            n.appendChild(doc.createTextNode(certificate))
            certificates.appendChild(n)

        scrubbers = doc.createElement('scrubbers')
        root.appendChild(scrubbers)
        for scrubber in self.scrubbers:
            n = doc.createElement('scrubber')
            n.appendChild(doc.createTextNode(scrubber))
            scrubbers.appendChild(n)

        postage = doc.createElement('postage')
        postage.appendChild(doc.createTextNode(self.postage))
        root.appendChild(postage)

        date = doc.createElement('date')
        date.appendChild(doc.createTextNode(self.date))
        root.appendChild(date)

        version = doc.createElement('version')
        version.appendChild(doc.createTextNode(self.version))
        root.appendChild(version)

        publickey = doc.createElement('publickey')
        publickey.appendChild(doc.createTextNode(self.publickey))
        root.appendChild(publickey)

        signature = doc.createElement('signature')
        signature.appendChild(doc.createTextNode(self.signature))
        root.appendChild(signature)

        return doc.toprettyxml("  ", "\n", "ISO-8859-1"), root, doc

    def from_xmlobj(self, root_node):
        if root_node is None:
            return
        try:
            for xsection in root_node.childNodes:
                if xsection.nodeType != Node.ELEMENT_NODE:
                    continue
                if xsection.tagName == 'contacts':
                    for xcontacts in xsection.childNodes:
                        for xcontact in xcontacts.childNodes:
                            if (xcontact.nodeType == Node.TEXT_NODE):
                                self.contacts.append(xcontact.wholeText.strip().encode())
                elif xsection.tagName == 'sources':
                    for xsources in xsection.childNodes:
                        for xsource in xsources.childNodes:
                            if (xsource.nodeType == Node.TEXT_NODE):
                                self.sources.append(xsource.wholeText.strip().encode())
                elif xsection.tagName == 'certificates':
                    for xcertificates in xsection.childNodes:
                        for xcertificate in xcertificates.childNodes:
                            if (xcertificate.nodeType == Node.TEXT_NODE):
                                self.certificates.append(xcertificate.wholeText.strip().encode())
                elif xsection.tagName == 'scrubbers':
                    for xscrubbers in xsection.childNodes:
                        for xscrubber in xscrubbers.childNodes:
                            if (xscrubber.nodeType == Node.TEXT_NODE):
                                self.scrubbers.append(xscrubber.wholeText.strip().encode())
                elif xsection.tagName == 'publickey':
                    for xkey in xsection.childNodes:
                        if (xkey.nodeType == Node.TEXT_NODE):
                            self.publickey=xkey.wholeText.strip().encode()
                elif xsection.tagName == 'postage':
                    for xpostage in xsection.childNodes:
                        if (xpostage.nodeType == Node.TEXT_NODE):
                            self.date=xpostage.wholeText.strip().encode()
                elif xsection.tagName == 'date':
                    for xkey in xsection.childNodes:
                        if (xkey.nodeType == Node.TEXT_NODE):
                            self.date=xkey.wholeText.strip().encode()
                elif xsection.tagName == 'version':
                    for xkey in xsection.childNodes:
                        if (xkey.nodeType == Node.TEXT_NODE):
                            self.version=xkey.wholeText.strip().encode()
                elif xsection.tagName == 'signature':
                    for xkey in xsection.childNodes:
                        if (xkey.nodeType == Node.TEXT_NODE):
                            self.signature=xkey.wholeText.strip().encode()
        except:
            dhnio.DprintException()

#-------------------------------------------------------------------------------

def makeDefaultIdentity(name='', ip='', successTCP='', successSSH='', successHTTP=''):
    dhnio.Dprint(4, 'identity.makeDefaultIdentity: %s %s tcp:%s ssh:%s http:%s' % (name, ip, successTCP, successSSH, successHTTP))
    if ip == '':
        ip = dhnio.ReadTextFile(settings.ExternalIPFilename())
    if name == '':
        name = ip.replace('.', '-') + '_' + time.strftime('%M%S')
    servername = settings.DefaultIdentityServer()
    url = 'http://'+servername+'/'+name.lower()+'.xml'

    ident = identity(xmlsrc=default_identity_src)
    ident.sources.append(url.encode("ascii").strip())
    ident.certificates=[]
    cdict = {}
    if settings.enableTCP():
        cdict['tcp'] = 'tcp://'+ip+':'+settings.getTCPPort()
    if settings.enableSSH():
        cdict['ssh'] = 'ssh://'+ip+':'+settings.getSSHPort()
    if settings.enableHTTP() or settings.enableHTTPServer():
        cdict['http'] = 'http://'+ip+':'+settings.getHTTPPort()
    if settings.enableQ2Q():
        if settings.getQ2Qusername().strip() != '' and settings.getQ2Qhost().strip() != '':
            cdict['q2q'] = 'q2q://'+settings.getQ2Quserathost()
        else:
            cdict['q2q'] = 'q2q://default@'+settings.getQ2Qhost()
    if settings.enableEMAIL():
        if settings.getEmailAddress().strip() != '':
            cdict['email'] = 'email://'+settings.getEmailAddress()
        else:
            cdict['email'] = 'email://'+name+'@datahaven.net'
    if settings.enableCSpace():
        if settings.getCSpaceKeyID() != '':
            cdict['cspace'] = 'cspace://'+settings.getCSpaceKeyID()
        else:
            cdict['cspace'] = 'cspace://not-registered'
        
    for c in misc.validTransports:
        if cdict.has_key(c):
            ident.contacts.append(cdict[c].encode("ascii").strip())

    ident.publickey = dhncrypto.MyPublicKey()
    ident.date = time.strftime('%b %d, %Y')
    ident.postage = "1"

    revnum = dhnio.ReadTextFile(settings.RevisionNumberFile())
    repo, location = misc.ReadRepoLocation()
    ident.version = (revnum.strip() + ' ' + repo.strip() + ' ' + dhnio.osinfo().strip()).strip()

    ident.sign()

    return ident

#-------------------------------------------------------------------------------

def test1():
    myidentity=misc.getLocalIdentity()
    print 'getIP =', myidentity.getIP()
    if myidentity.Valid():
        print "myidentity is Valid!!!!"
    else:
        print "myidentity is not Valid"
        misc.saveLocalIdentity()            # sign and save
        raise Exception("myidentity is not Valid")
    print "myidentity.contacts"
    print myidentity.contacts
    print "len myidentity.contacts "
    print len (myidentity.contacts)
    print "len myidentity.contacts[0] "
    print myidentity.contacts[0]
    con=myidentity.getContact()
    print "con:", con, type(con)
    protocol, machine, port, filename = nameurl.UrlParse(con)
    print protocol, machine, port, filename
    print "identity.main serialize:\n", myidentity.serialize()
    for index in range(myidentity.getContactsNumber()):
        proto, host, port, filename = myidentity.getContactParts(index)
        print '[%s] [%s] [%s] [%s]' % (proto, host, port, filename)


def test2():
    ident = makeDefaultIdentity()
    print ident.serialize() 

#------------------------------------------------------------------------------ 


def main():
    misc.loadLocalIdentity()
    if misc.isLocalIdentityReady():
        misc.getLocalIdentity().sign()
        print misc.getLocalIdentity().serialize()
        print 'Valid is: ', misc.getLocalIdentity().Valid()
    else:
        misc.setLocalIdentity(makeDefaultIdentity(sys.argv[1]))
        misc.saveLocalIdentity()
        print misc.getLocalIdentity().serialize()
        print 'Valid is: ', misc.getLocalIdentity().Valid()
        misc._LocalIdentity = None
        misc.loadLocalIdentity()        
                              
#------------------------------------------------------------------------------ 


if __name__ == '__main__':
    dhnio.SetDebug(18)
    main()






#!/usr/bin/env python
#identity_restorer.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in identity_restorer.py')
from twisted.internet.defer import Deferred, DeferredList, maybeDeferred
from twisted.internet.task import LoopingCall


from lib.automat import Automat
import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.identitycache as identitycache
import lib.identity as identity
import lib.dhncrypto as dhncrypto
import lib.dhnnet as dhnnet


import installer
import lib.automats as automats

import webcontrol

_IdentityRestorer = None
_WorkingIDURL = ''
_WorkingKey = ''

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _IdentityRestorer
    if _IdentityRestorer is None:
        _IdentityRestorer = IdentityRestorer('identity_restorer', 'READY', 4)
    if event is not None:
        _IdentityRestorer.automat(event, arg)
    return _IdentityRestorer


class IdentityRestorer(Automat):
    MESSAGES = {
        'MSG_01':   ['download central server identity'], 
        'MSG_02':   ['network connection failed', 'red'],
        'MSG_03':   ['download my identity'],
        'MSG_04':   ['identity not exist', 'red'],
        'MSG_05':   ['verifying my identity and private key'],
        'MSG_06':   ['my account were restored? cool!', 'green'], }
    
    def msg(self, arg): 
        msg = self.MESSAGES.get(arg, ['', 'black'])
        text = msg[0]
        color = 'black'
        if len(msg) == 2:
            color = msg[1]
        return text, color
    
    def state_changed(self, oldstate, newstate):
        automats.set_global_state('ID_RESTORE ' + newstate)
        installer.A('identity_restorer.state', newstate)

    def A(self, event, arg):
        #---READY---
        if self.state == 'READY':
            if event == 'start':
                installer.A('print', self.msg('MSG_01'))
                self.doSetWorkingIDURL(arg)
                self.doSetWorkingKey(arg)
                self.doRequestCentralIdentity()
                self.state = 'CENTRAL_ID'
        #---CENTRAL_ID---
        elif self.state == 'CENTRAL_ID':
            if event == 'central-id-failed':
                installer.A('print', self.msg('MSG_02'))
                self.doClearWorkingIDURL()
                self.doClearWorkingKey()
                self.state = 'READY'
            elif event == 'central-id-received':
                installer.A('print', self.msg('MSG_03'))
                self.doRequestMyIdentity()
                self.state = 'MY_ID' 
        #---MY_ID---
        elif self.state == 'MY_ID':
            if event == 'my-id-failed':
                installer.A('print', self.msg('MSG_04'))
                self.doClearWorkingIDURL()
                self.doClearWorkingKey()
                self.state = 'READY'
            elif event == 'my-id-received':
                installer.A('print', self.msg('MSG_05'))
                self.doVerifyAndRestore(arg)
                self.state = 'WORK'                 
        #---WORK---
        elif self.state == 'WORK':
            if event == 'restore-failed':
                installer.A('print', arg)
                self.doClearWorkingIDURL()
                self.doClearWorkingKey()
                self.state = 'READY'
            elif event == 'restore-success':
                installer.A('print', self.msg('MSG_06'))
                self.doRestoreSave()
                self.state = 'DONE' 
        #---DONE---
        elif self.state == 'DONE':
            pass
                
    def doSetWorkingIDURL(self, arg):
        global _WorkingIDURL
        _WorkingIDURL = arg['idurl']
        
    def doSetWorkingKey(self, arg):
        global _WorkingKey
        _WorkingKey = arg['keysrc']

    def doClearWorkingIDURL(self):
        global _WorkingIDURL
        _WorkingIDURL = ''
        
    def doClearWorkingKey(self):
        global _WorkingKey
        _WorkingKey = ''

    def doRequestCentralIdentity(self):
        dhnio.Dprint(4, 'identity_restorer.doRequestCentralIdentity')
        identitycache.immediatelyCaching(settings.CentralID()).addCallbacks(
            lambda x: self.automat('central-id-received'),
            lambda x: self.automat('central-id-failed'))
        
    def doRequestMyIdentity(self):
        global _WorkingIDURL
        idurl = _WorkingIDURL
        dhnio.Dprint(4, 'identity_restorer.doRequestMyIdentity %s' % idurl)
        dhnnet.getPageTwisted(idurl).addCallbacks(
            lambda src: self.automat('my-id-received', src),
            lambda err: self.automat('my-id-failed', err))
    
    def doVerifyAndRestore(self, arg):
        global _WorkingKey
        dhnio.Dprint(4, 'identity_restorer.doVerifyAndRestore')
    
        remote_identity_src = arg

        if os.path.isfile(settings.KeyFileName()):
            dhnio.Dprint(4, 'identity_restorer.doVerifyAndRestore will backup and remove ' + settings.KeyFileName())
            dhnio.backup_and_remove(settings.KeyFileName())

        if os.path.isfile(settings.LocalIdentityFilename()):    
            dhnio.Dprint(4, 'identity_restorer.doVerifyAndRestore will backup and remove ' + settings.LocalIdentityFilename())
            dhnio.backup_and_remove(settings.LocalIdentityFilename())
    
        try:
            remote_ident = identity.identity(xmlsrc = remote_identity_src)
            local_ident = identity.identity(xmlsrc = remote_identity_src)
        except:
            dhnio.DprintException()
            self.automat('restore-failed', 'remote identity have incorrect format' )
            return
    
        try:
            res = remote_ident.Valid()
        except:
            dhnio.DprintException()
            res = False
        if not res:
            self.automat('restore-failed', 'remote_identity_not_valid')
            return
    
        dhncrypto.ForgetMyKey()
        dhnio.WriteFile(settings.KeyFileName(), _WorkingKey)
        try:
            dhncrypto.InitMyKey()
        except:
            dhnio.DprintException()
            try:
                os.remove(settings.KeyFileName())
            except:
                pass
            self.automat('restore-failed', 'private key not valid')
            return
    
        try:
            local_ident.sign()
        except:
            dhnio.DprintException()
            self.automat('restore-failed', 'error while signing identity')
            return
    
        if remote_ident.signature != local_ident.signature:
            self.automat('restore-failed', 'signature do not match')
            return
    
        misc.setLocalIdentity(local_ident)
        misc.saveLocalIdentity()
    
        dhnio.WriteFile(settings.UserNameFilename(), misc.getIDName())
    
        if os.path.isfile(settings.KeyFileName()+'.backup'):
            dhnio.Dprint(4, 'identity_restorer.doVerifyAndRestore will remove backup file for ' + settings.KeyFileName())
            dhnio.remove_backuped_file(settings.KeyFileName())

        if os.path.isfile(settings.LocalIdentityFilename()+'.backup'):
            dhnio.Dprint(4, 'identity_restorer.doVerifyAndRestore will remove backup file for ' + settings.LocalIdentityFilename())
            dhnio.remove_backuped_file(settings.LocalIdentityFilename())

        self.automat('restore-success')
        
    def doRestoreSave(self):
        settings.uconfig().set('central-settings.desired-suppliers', '0')
        settings.uconfig().set('central-settings.needed-megabytes', '0Mb')
        settings.uconfig().set('central-settings.shared-megabytes', '0Mb')
        settings.uconfig().update()









# memdebug.py

import cherrypy
import dowser

def start(port):
    cherrypy.config.update({
        'environment': 'embedded',
        'server.socket_port': port,
        'server.socket_host': '0.0.0.0',
    })
    cherrypy.tree.mount(dowser.Root())

    #cherrypy.server.quickstart()
    cherrypy.engine.start()

def stop():
    cherrypy.engine.exit()


#from twisted.web.wsgi import WSGIResource
#from twisted.internet import reactor
#
#from dozer import Dozer
#
#def application(environ, start_response):
#    start_response('200 OK', [('Content-type', 'text/plain')])
#    return ['Hello, world!']
#
#def start():
#    resource = WSGIResource(reactor, reactor.getThreadPool(), application)
#    wsgi_app = Dozer(application)





#!/usr/bin/env python
#automats.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in automat.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall
from twisted.internet import threads

from lib.automat import Automat
import lib.automats as automats

import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.stun as stun
import lib.dhnnet as dhnnet
import lib.transport_control as transport_control
if transport_control._TransportCSpaceEnable:
    import lib.transport_cspace as transport_cspace

import p2p_connector
import shutdowner
import run_upnpc


_NetworkConnector = None

#------------------------------------------------------------------------------

def A(event=None, arg=None):
    global _NetworkConnector
    if _NetworkConnector is None:
        _NetworkConnector = NetworkConnector('network_connector', 'AT_STARTUP', 4)
    if event is not None:
        _NetworkConnector.automat(event, arg)
    return _NetworkConnector


class NetworkConnector(Automat):
    timers = {'timer-1min':  (60,    ['DISCONNECTED']),
              'timer-1hour': (60*60, ['CONNECTED',])}

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('NETWORK ' + newstate)
        p2p_connector.A('network_connector.state', newstate)

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'init':
                self.doInit()
                self.doStun()
                self.state = 'STUN'
        #---STUN---
        elif self.state == 'STUN':
            if event == 'stun-success' and not self.isUPNP():
                self.state = 'CONNECTED'
            elif event == 'stun-success' and self.isUPNP():
                self.doUPNP()
                self.state = 'UPNP'
            elif event == 'stun-failed':
                self.state = 'DISCONNECTED'
        #---UPNP---
        elif self.state == 'UPNP':
            if event == 'upnp-done':
                self.state = 'CONNECTED'
        #---CONNECTED---
        elif self.state == 'CONNECTED':
            if ( event == 'cspace-status' and arg in ['offline', 'disconnecting'] ) or ( event == 'timer-1hour' and not self.isTrafficIN() ):
                self.doStun()
                self.state = 'STUN'
        #---DISCONNECTED---
        elif self.state == 'DISCONNECTED':
            if event == 'timer-1min' or ( event == 'cspace-status' and arg == 'online' ):
                self.doCallGoogle()
                self.state = 'GOOGLE?'
        #---GOOGLE?---
        elif self.state == 'GOOGLE?':
            if event == 'google-success':
                self.doStun()
                self.state = 'STUN'
            elif event == 'google-failed':
                self.state = 'DISCONNECTED'

    def isIPexternal(self):
        externalip = misc.readExternalIP()
        localip = misc.readLocalIP()
        return localip != externalip

    def isUPNP(self):
        return settings.enableUPNP()

    def isTrafficIN(self):
        return transport_control.TimeSinceLastReceive() < 60*10

    def doInit(self):
        if transport_control._TransportCSpaceEnable:
            transport_cspace.SetStatusNotifyFunc(cspace_status_changed)

    def doStun(self):
        def stun_success(externalip):
            if externalip == '0.0.0.0':
                self.automat('stun-failed')
                return
            localip = dhnnet.getLocalIp()
            dhnio.WriteFile(settings.ExternalIPFilename(), str(externalip))
            dhnio.WriteFile(settings.LocalIPFilename(), str(localip))
            self.automat('stun-success')
        def stun_failed(x):
            self.automat('stun-failed')
        stun.stunExternalIP(
            close_listener=False, 
            internal_port=settings.getUDPPort(),
            verbose=True if dhnio.Debug(10) else False).addCallbacks(stun_success, stun_failed)

    def doUPNP(self):
        UpdateUPNP()

    def doCallGoogle(self):
        if dhnnet.TestInternetConnection():
            self.automat('google-success')
        else:
            self.automat('google-failed')


def UpdateUPNP():
    #global _UpnpResult
    dhnio.Dprint(8, 'network_connector.UpdateUPNP ')

#    protos_need_upnp = set(['tcp', 'ssh', 'http'])
    protos_need_upnp = set(['tcp',])

    #we want to update only enabled protocols
    if not settings.enableTCP():
        protos_need_upnp.discard('tcp')
    if not settings.enableSSH() or not transport_control._TransportSSHEnable:
        protos_need_upnp.discard('ssh')
    if not settings.enableHTTPServer() or not transport_control._TransportHTTPEnable:
        protos_need_upnp.discard('http')

    dhnio.Dprint(6, 'network_connector.UpdateUPNP want to update protocols: ' + str(protos_need_upnp))

    def _update_next_proto():
        if len(protos_need_upnp) == 0:
            #dhnio.Dprint(4, 'network_connector.update_upnp done: ' + str(_UpnpResult))
            A('upnp-done')
            return
        dhnio.Dprint(14, 'network_connector.UpdateUPNP._update_next_proto ' + str(protos_need_upnp))
        proto = protos_need_upnp.pop()
        protos_need_upnp.add(proto)
        if proto == 'tcp':
            port = settings.getTCPPort()
        elif proto == 'ssh':
            port = settings.getSSHPort()
        elif proto == 'http':
            port = settings.getHTTPPort()
        d = threads.deferToThread(_call_upnp, port)
        d.addCallback(_upnp_proto_done, proto)

    def _call_upnp(port):
        # start messing with upnp settings
        # success can be false if you're behind a router that doesn't support upnp
        # or if you are not behind a router at all and have an external ip address
        shutdowner.A('block')
        success, port = run_upnpc.update(port)
        shutdowner.A('unblock')
        return (success, port)

    def _upnp_proto_done(result, proto):
        dhnio.Dprint(4, 'network_connector.UpdateUPNP._upnp_proto_done %s: %s' % (proto, str(result)))
        #_UpnpResult[proto] = result[0]
        #if _UpnpResult[proto] == 'upnp-done':
        if result[0] == 'upnp-done':
            if proto == 'tcp':
                settings.setTCPPort(result[1])
            elif proto == 'ssh':
                settings.setSSHPort(result[1])
            elif proto == 'http':
                settings.setHTTPPort(result[1])
        protos_need_upnp.discard(proto)
        reactor.callLater(0, _update_next_proto)

    _update_next_proto()


def cspace_status_changed(status):
    dhnio.Dprint(4, 'network_connector.cspace_status_changed [%s]' % status.upper())
    A('cspace-status', status)





#!/usr/bin/python
#dhninit.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in dhninit.py')
from twisted.internet.defer import Deferred,  DeferredList
from twisted.internet import task

import lib.dhnio as dhnio


# need to change directory in case we were not in the p2p directory when datahaven started up,
# need it to find dhntester.py and potentially others
#try:
#    os.chdir(os.path.abspath(os.path.dirname(__file__)))
#except:
#    pass

UImode = ''

#------------------------------------------------------------------------------

def run(UI='', options=None, args=None, overDict=None):
    init(UI, options, args, overDict)


def init_local(UI=''):
    global UImode
    UImode = UI
    dhnio.Dprint(2, "dhninit.init_local")

    import lib.settings as settings
    import lib.misc as misc
    misc.init()
    misc.UpdateSettings()

    #Here we can change users settings depending on user name
    #Small hack, but we want to do only during testing period.
    settings_patch()

    import lib.commands as commands
    commands.init()

    if sys.argv.count('--twisted'):
        class MyTwistedOutputLog:
            softspace = 0
            def read(self): pass
            def write(self, s):
                dhnio.Dprint(0, s.strip())
            def flush(self): pass
            def close(self): pass
        from twisted.python import log as twisted_log
        twisted_log.startLogging(MyTwistedOutputLog(), setStdout=0)
#    import twisted.python.failure as twisted_failure
#    twisted_failure.startDebugMode()
#    twisted_log.defaultObserver.stop()

    from twisted.internet import defer
    defer.setDebugging(True)

    if settings.enableWebStream():
        misc.StartWebStream()

    if settings.enableWebTraffic():
        misc.StartWebTraffic()
        
    if settings.enableMemoryProfile():
        try:
            from guppy import hpy
            hp = hpy()
            hp.setrelheap()
            dhnio.Dprint(2, 'hp.heap():\n'+str(hp.heap()))
            dhnio.Dprint(2, 'hp.heap().byrcs:\n'+str(hp.heap().byrcs))
            dhnio.Dprint(2, 'hp.heap().byvia:\n'+str(hp.heap().byvia))
            import guppy.heapy.RM
        except:
            dhnio.Dprint(2, "dhninit.init_local guppy package is not installed")            

    import lib.tmpfile as tmpfile
    tmpfile.init(settings.getTempDir())

    import lib.dhnnet as dhnnet
    settings.update_proxy_settings()

    import run_upnpc
    run_upnpc.init()

    import lib.dhncrypto as dhncrypto
    import lib.identity as identity

    import webcontrol
    import lib.automats as automats
    webcontrol.GetGlobalState = automats.get_global_state

    import dhnupdate
    dhnupdate.SetNewVersionNotifyFunc(webcontrol.OnGlobalVersionReceived)

    start_logs_rotate()

def init_contacts(callback=None, errback=None):
    dhnio.Dprint(2, "dhninit.init_contacts")

    import lib.misc as misc
    misc.loadLocalIdentity()
    if misc._LocalIdentity is None:
        if errback is not None:
            errback(1)
        return

    import lib.contacts as contacts
    contacts.init()

    import lib.identitycache as identitycache
    identitycache.init(callback, errback)


def init_connection():
    global UImode
    dhnio.Dprint(2, "dhninit.init_connection")

    import webcontrol

    import contacts_status
    contacts_status.init()

    import central_service
    central_service.OnListSuppliersFunc = webcontrol.OnListSuppliers
    central_service.OnListCustomersFunc = webcontrol.OnListCustomers

    import customerservice
    customerservice.init()

    import money

    import message
    message.init()
    message.OnIncommingMessageFunc = webcontrol.OnIncommingMessage

    import lib.automats as automats
    automats.SetGlobalStateNotifyFunc(webcontrol.OnGlobalStateChanged)
    automats.SetSingleStateNotifyFunc(webcontrol.OnSingleStateChanged)

    import identitypropagate
    identitypropagate.init()

    try:
        from dhnicon import USE_TRAY_ICON
    except:
        USE_TRAY_ICON = False
        dhnio.DprintException()

    if USE_TRAY_ICON:
        import dhnicon
        dhnicon.SetControlFunc(webcontrol.OnTrayIconCommand)

    #init the mechanism for sending and requesting files for repairing backups
    import io_throttle
    io_throttle.init()

    import backup_db
    backup_db.init()

    import backups
    backups.init()
    backups.SetBackupStatusNotifyCallback(webcontrol.OnBackupStats)
    backups.SetLocalFilesNotifyCallback(webcontrol.OnReadLocalFiles)
    
    #backup_monitor.init()
    #backup_monitor.OnIncomingListFilesFunc = webcontrol.OnIncomingListFiles
    #backup_monitor.OnIncomingDataPacketFunc = webcontrol.OnIncomingDataPacket
    #backup_monitor.OnNewBackupListFilesFunc = webcontrol.OnNewBackupListFiles
    #backup_monitor.SetStatusCallBackForGuiBackup(webcontrol.OnBackupStatus)

    import restore_monitor
    restore_monitor.init()
    restore_monitor.OnRestorePacketFunc = webcontrol.OnRestoreProcess
    restore_monitor.OnRestoreDoneFunc = webcontrol.OnRestoreDone

#    import summary
#    summary.init()

#    webcontrol.ready()

#    if UImode.lower() == 'show':
#        wcinit.addCallback(webcontrol.show)
#        wcinit.addErrback(lambda x: dhnio.DprintException())


def init_modules():
    dhnio.Dprint(2,"dhninit.init_modules")

    import webcontrol

    import localtester
    import backupshedule
    #import ratings
    import fire_hire
    #import backup_monitor
    import dhnupdate

    #reactor.callLater(3, backup_monitor.start)

    reactor.callLater(5, localtester.init)

    reactor.callLater(10, backupshedule.init)

    #reactor.callLater(20, ratings.init)

    #reactor.callLater(25, firehire.init)

    reactor.callLater(15, dhnupdate.init)

    # finally we can decrease our priority
    # because during backup DHN eats too much CPU
    dhnio.LowerPriority()

    webcontrol.OnInitFinalDone()


def shutdown(x=None):
    global initdone
    dhnio.Dprint(2, "dhninit.shutdown " + str(x))
    dl = []

    import lib.stun
    dl.append(lib.stun.stopUDPListener())
#    import backup_monitor
#    backup_monitor.shutdown()
#    
#    import p2p_connector
#    p2p_connector.shutdown()

    import dobackup
    dobackup.shutdown()

    import backups
    backups.shutdown()

#    import fire_hire
#    fire_hire.shutdown()

    import ratings
    ratings.shutdown()

    import contacts_status
    contacts_status.shutdown()

    import run_upnpc
    run_upnpc.shutdown()

    import localtester
    localtester.shutdown()

    import webcontrol
    dl.append(webcontrol.shutdown())

    import lib.transport_control as transport_control
    dl.append(transport_control.shutdown())

    import lib.weblog as weblog
    weblog.shutdown()

    initdone = False

    return DeferredList(dl)


def shutdown_restart(param=''):
    dhnio.Dprint(2, "dhninit.shutdown_restart ")

    def do_restart(param):
        import lib.misc as misc
        misc.DoRestart(param)

    def shutdown_finished(x, param):
        dhnio.Dprint(2, "dhninit.shutdown_restart.shutdown_finished want to stop the reactor")
        reactor.addSystemEventTrigger('after','shutdown', do_restart, param)
        reactor.stop()

    d = shutdown('restart')
    d.addBoth(shutdown_finished, param)


def shutdown_exit(x=None):
    dhnio.Dprint(2, "dhninit.shutdown_exit ")

    def shutdown_reactor_stop(x=None):
        dhnio.Dprint(2, "dhninit.shutdown_exit want to stop the reactor")
        reactor.stop()

    d = shutdown(x)
    d.addBoth(shutdown_reactor_stop)


def settings_patch():
    dhnio.Dprint(6, 'dhninit.settings_patch ')
    #import lib.settings as settings
    #settings.enableCSpace(True)
    #settings.setUPNPatStartup(True)
    #settings.enableQ2Q(False)
    #settings.enableHTTP(False)
    #settings.enableHTTPServer(False)

#    import lib.misc as misc
#    settings.enableQ2Q(True)    #small hook to switch on the q2q

#    if not misc.getIDName() in ['dhncentral', 'petarpenev', 'veseleeypc', 'ekaterina', 'veselin']:
#        settings.enableQ2Q(False)

#I want to rename the icon name to DataHaven
#Because it is too long and looks ugly on 2 lines
#    misc.removeWindowsShortcut('DataHaven.Net.lnk')
#    misc.removeStartMenuShortcut('DataHaven.Net.lnk')
#    misc.removeWindowsShortcut('DataHaven.lnk')
#    misc.removeStartMenuShortcut('DataHaven.lnk')
#    misc.removeWindowsShortcut('Data Haven.lnk')
#    misc.removeStartMenuShortcut('Data Haven.lnk')
#    misc.removeWindowsShortcut('Data Haven Net.lnk')
#    misc.removeStartMenuShortcut('Data Haven Net.lnk')

#    if misc.getIDName() in ['nonsons', 'van4eg', 'kirill']:
#    settings.uconfig().set('transport.transport-q2q.transport-q2q-enable', 'True')
#    settings.uconfig().set('updates.updates-shedule', '4\n12:00:00\n1\n\n')

#    if settings.uconfig('transport.transport-q2q.transport-q2q-host') != 'datahaven.net':
#        settings.uconfig().set('transport.transport-q2q.transport-q2q-host', 'datahaven.net')
#        settings.uconfig().set('transport.transport-q2q.transport-q2q-username', '')
#        settings.uconfig().set('transport.transport-q2q.transport-q2q-password', '')

    #import lib.misc as misc
    #if misc.getLocalIdentity().getProtoContact('cspace') is None:
        #dhnio.Dprint(4, 'dhninit.settings_patch want to enable cspace')

        #import lib.transport_cspace as transport_cspace
        #reactor.callLater(60*2, transport_cspace.install)
#        def cspace_init_done(x):
#            dhnio.Dprint(4, 'dhninit.settings_patch cspace init was done')
#            def cspace_register_done(x):
#                dhnio.Dprint(4, 'dhninit.settings_patch cspace registration done')
#                settings.enableCSpace(True)
#                settings.uconfig().update()
#            transport_cspace.register(misc.getIDName()).addCallback(cspace_register_done)
#        transport_cspace.init().addCallback(cspace_init_done)
    import lib.settings as settings
    settings.enableUDP(True)
    settings.enableCSpace(False)
    

def start_logs_rotate():
    dhnio.Dprint(4, 'dhninit.start_logs_rotate')
    def erase_logs():
        dhnio.Dprint(4, 'dhninit.erase_logs ')
        import lib.settings as settings
        logs_dir = settings.LogsDir()
        total_sz = 0
        for filename in os.listdir(logs_dir):
            filepath = os.path.join(logs_dir, filename)
            if not filename.endswith('.log'):
                # this is not a log file - we did not create it - do nothing
                continue
            if filename.startswith('dhnmain-'):
                # remove "old version" files, now we have files started with "dhn-" 
                try:
                    os.remove(os.path.join(logs_dir, filename))
                    dhnio.Dprint(6, 'dhninit.erase_logs %s was deleted because it is old version' % filename)
                except:
                    dhnio.Dprint(1, 'dhninit.erase_logs ERROR can not remove ' + filename)
                continue
            # count the total size of the all log files
            try:
                file_size = os.path.getsize(os.path.join(logs_dir, filename))
            except:
                file_size = 0
            total_sz += file_size 
            # if the file is bigger than 2mb - erase it
            if file_size > 1024*1024*2:
                if os.access(filepath, os.W_OK):
                    try:
                        os.remove(filepath)
                        dhnio.Dprint(6, 'dhninit.erase_logs %s was deleted because it is too big' % filename)
                    except:
                        dhnio.Dprint(1, 'dhninit.erase_logs ERROR can not remove ' + filename)
            # if this is a file for every execution (started with "dhn-") ... 
            if filename.startswith('dhn-'):
                # we check if file is writable - so we can remove it
                if not os.access(filepath, os.W_OK):
                    continue
                # get its datetime
                try:
                    dtm = time.mktime(time.strptime(filename[4:-4],'%y%m%d%H%M%S'))
                except:
                    dhnio.DprintException()
                    continue          
                # we want to check if it is more than 10 days old ...
                if time.time() - dtm > 60*60*24*10: 
                    try:
                        os.remove(filepath)
                        dhnio.Dprint(6, 'dhninit.erase_logs %s was deleted because it is old' % filename)
                    except:
                        dhnio.Dprint(1, 'dhninit.erase_logs ERROR can not remove ' + filename)
                    continue
                # also we want to check if all those files are too big - 5 MB is enough
                if total_sz > 1024*1024*5:
                    try:
                        os.remove(filepath)
                        dhnio.Dprint(6, 'dhninit.erase_logs %s was deleted because log files is too big' % filename)
                    except:
                        dhnio.Dprint(1, 'dhninit.erase_logs ERROR can not remove ' + filename)
                    continue
                     
    task.LoopingCall(erase_logs).start(60*60*24)



def check_install():
    dhnio.Dprint(2, 'dhninit.check_install ')
    import lib.settings as settings
    import lib.identity as identity
    import lib.dhncrypto as dhncrypto

    keyfilename = settings.KeyFileName()
    idfilename = settings.LocalIdentityFilename()
    if not os.path.exists(keyfilename) or not os.path.exists(idfilename):
        dhnio.Dprint(2, 'dhninit.check_install local key or local id not exists')
        return False

    current_key = dhnio.ReadBinaryFile(keyfilename)
    current_id = dhnio.ReadBinaryFile(idfilename)

    if current_id == '':
        dhnio.Dprint(2, 'dhninit.check_install local identity is empty ')
        return False

    if current_key == '':
        dhnio.Dprint(2, 'dhninit.check_install private key is empty ')
        return False

    try:
        dhncrypto.InitMyKey()
    except:
        dhnio.Dprint(2, 'dhninit.check_install fail loading private key ')
        return False

    try:
        ident = identity.identity(xmlsrc=current_id)
    except:
        dhnio.Dprint(2, 'dhninit.check_install fail init local identity ')
        return False

    try:
        res = ident.Valid()
    except:
        dhnio.Dprint(2, 'dhninit.check_install wrong data in local identity   ')
        return False

    if not res:
        dhnio.Dprint(2, 'dhninit.check_install local identity is not valid ')
        return False

    dhnio.Dprint(2, 'dhninit.check_install done')
    return True





#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# This file is just for things that users may need to set differently for their machine.
# We just want things that are really worth exposing to a user, not everything.

import os


import userconfig
import dhnio
import eccmap
import dhnmath
import diskspace
import nameurl


_UserConfig = None
_BaseDirPath = ''
_InitDone = False
_OverrideDict = {}
_BandwidthLimit = None

#-------------------------------------------------------------------------------


# base_dir - location for ".datahaven" folder
# base_dir=None - will start in default location:
# Windows: os.environ.get('APPDATA')\DataHaven.NET
# or:      os.path.expanduser('~')\Application Data\DataHaven.NET
# because on WindowsXP: c:\\Document and Settings\\[user]\\Application Data\\DataHaven.NET
#      but on Windows7: c:\\Users\\[user]\\AppData\\Roaming\\DataHaven.NET
# Linux: os.path.expanduser('~')/datahavennet
# possible move to ~/.DataHaven.NET ???
def init(base_dir=None):
    global _InitDone
    if _InitDone:
        return
    _InitDone = True
    _init(base_dir)

#------------------------------------------------------------------------------

def uconfig(key=None):
    global _UserConfig
    global _OverrideDict
    #init()
    if _UserConfig is None:
        if os.path.exists(os.path.join(MetaDataDir(),"user-config")) and not os.path.exists(os.path.join(MetaDataDir(),"userconfig")):
            dhnio.Dprint(4, 'settings.uconfig rename "user-config" to "userconfig"')
            try:
                os.rename(os.path.join(MetaDataDir(),"user-config"), os.path.join(MetaDataDir(),"userconfig"))
            except:
                pass
        dhnio.Dprint(6, 'settings.uconfig loading user configuration from: ' + UserConfigFilename())
        _UserConfig = userconfig.UserConfig(UserConfigFilename())
    if key is None:
        return _UserConfig
    else:
        if key in _OverrideDict:
            return _OverrideDict[key]
        res = _UserConfig.get(key)
        if res is None:
            return ''
        return res

# this (key, value) pairs will be used instead values from  _UserConfig
def override(key, value):
    global _OverrideDict
    dhnio.Dprint(4, 'settings.override %s=%s' % (key, value))
    _OverrideDict[key] = value

def override_dict(d):
    for key, value in d.items():
        override(key, value)

#------------------------------------------------------------------------------

def BasePricePerGBmonth():
    return 0.1

def BasePricePerGBDay():
    return BasePricePerGBmonth() / 30.0

def defaultDebugLevel():
    return 4

def IntSize():
    return 4

def MinimumSendingDelay():
    return 0.01

def MaximumSendingDelay():
    return 4.0

# Should be a function of disk space available  PREPRO
def MaxPacketsOutstanding():
    return 100

# What size DHNpackets we want
def PacketSizeTarget():           # BlockSizeTarget is this times eccmap.nodes()
    #return 50000
    #return 1024*99
    #return 500000
    #return (1024*1023) + 512 # :-)
    #return 1024 * 511 + 256 + 128 + 64 + 32 + 16 + 8 + 4 + 2
    #return 1024 * 256
    return 512 * 1024

# if sending below this speed - we count this supplier as failed
# if we sending too slow to all nodes - it's our problems
def SendingSpeedLimit():
    return 4 * 1024 # 1KB/sec is about 3.5MB/hour

# in kilo bytes per second
def DefaultBandwidthLimit():
    return 1024

def SendTimeOut():
    return 600                   # 600 seconds send timeout for ssh/tcp/q2q
    #return 60                     # 60 seconds send timeout for ssh/tcp/q2q
    #return 30

def MaxRetries():                 # "exponential backoff" - double each time so 1024*SendTimeOut seconds or 7 days
#    return(10)
    return 3                      # Veselin put only 3 retries (3*SendTimeOut) in thought we do not need too old packets

def DefaultSendTimeOutEmail():
    return 300                    # timeout for email sending

def DefaultSendTimeOutHTTP():
    return 60                     # timeout for http sending

def DefaultAlivePacketTimeOut():
    return 60 * 60                  #lets send alive packets every hour

def DefaultBandwidthReportTimeOut():
    return 60 * 60 * 24                #send BandwidthReport every 24 hours

def DefaultNeedSuppliersPacketTimeOut():
    return 60                     #if we need suppliers we will send it every 60 sec.

def DefaultSuppliersNumber():
    return 4

def DefaultDesiredSuppliers():
    return 4

def DefaultLocaltesterLoop():
    return 20                   #every 20 sec

def DefaultLocaltesterValidateTimeout():
    return 120 * 60               #every 120 min

def DefaultLocaltesterUpdateCustomersTimeout():
    return 5 * 60                 #every 5 min

def DefaultSignerSendingDelay():
    return 0.05

def DefaultSignerReceivingDelay():
    return 0.1

def DefaultSignerPingDelay():
    return 1.0

def MinimumUsernameLength():
    return 3

def MaximumUsernameLength():
    return 20

def DefaultDonatedMb():
    return 8*1024

def DefaultNeededMb():
    return 4*1024

def MinimumNeededMb():
    return 64

def MinimumDonatedMb():
    return 128

def CentralKillNotAliveUserAfter():
    return 60 #kill user after X days

def FireHireMinimumDelay():
    return 60 * 15 # do not want to fire suppliers too often, 15 minutes interval 

#-------------------------------------------------------------------------------

def ApplicationName():
    return 'DataHaven.NET'

def CentralID():
    return 'http://identity.datahaven.net/dhncentral.xml'

def MoneyServerID():
    #return 'http://identity.datahaven.net/wwww.xml'
    return 'http://identity.datahaven.net/offshore.xml'

def ListFilesFormat():         #  argument to ListFiles command to say format to return the data in
    return("Summary")          #  "Text", "Compressed", "Summary"

def DefaultEccMapName():
    return 'ecc/2x2'

def HMAC_key_word():
    return 'Vince+Veselin+Derek=BigMoneyAndSuccess'

def DefaultRepo(): 
    return 'devel'

def UpdateLocationURL(repo=DefaultRepo()):
    if repo == 'devel':
        return 'http://datahaven.net/repo/devel/'
    elif repo == 'stable':
        return 'http://datahaven.net/repo/stable/'
    else: 
        return 'http://identity.datahaven.net/downloads/'

def CurrentVersionDigestsFilename():
    return 'version.txt'

def FilesDigestsFilename():
    return 'info.txt'

def UpdateFolder():
    return 'windows/'

def CurrentVersionURL(repo='test'):
    return UpdateLocationURL(repo) + 'version.txt'

def LegalUsernameChars():
    return set("abcdefghijklmnopqrstuvwxyz0123456789-_")

def NotRealUsers():
    return ('http://identity.datahaven.net/cate2gpa.xml',
            'http://identity.datahaven.net/dcdemo.xml',
            'http://identity.datahaven.net/dctest23.xml',
            'http://identity.datahaven.net/derekcate.xml',
            'http://identity.datahaven.net/derekcatewin.xml',
            'http://identity.datahaven.net/dhncentral.xml',
            'http://identity.datahaven.net/ethan.xml',
            'http://identity.datahaven.net/ekaterina.xml',
            'http://identity.datahaven.net/gazebo.xml',
            'http://identity.datahaven.net/gazebo4.xml',
            'http://identity.datahaven.net/guesthouse.xml',
            'http://identity.datahaven.net/hcate2.xml',
            'http://identity.datahaven.net/mediapc2.xml',
            'http://identity.datahaven.net/offshore.xml',
            'http://identity.datahaven.net/terynlinux.xml',
            'http://identity.datahaven.net/terynvista.xml',
            'http://identity.datahaven.net/veseleeypc.xml',
            'http://identity.datahaven.net/veselin-macos.xml',
            'http://identity.datahaven.net/veselin.xml',
            'http://identity.datahaven.net/veselinux.xml',
            'http://identity.datahaven.net/veselonflash.xml',
            'http://identity.datahaven.net/vinceworkstation.xml',
            'http://identity.datahaven.net/vista3.xml',
            'http://identity.datahaven.net/workoffshoreai.xml',)

#-------------------------------------------------------------------------------

def BaseDirDefault():
    return os.path.join(os.path.expanduser('~'), '.datahaven')

# PREPRO - all of the paths below should be under some base directory
def BaseDirLinux():
    new_path = os.path.join(os.path.expanduser('~'), '.datahaven')
    if os.path.isdir(new_path):
        return new_path
    old_path = os.path.join(os.path.expanduser('~'), 'datahavennet')
    if os.path.isdir(old_path):
        return old_path
    return new_path

def BaseDirWindows():
    if not dhnio.Windows():
        return BaseDirDefault()
    #return os.path.join(os.path.expanduser('~'), 'Application Data', 'DataHavenNet')
    default_path = os.path.join(os.path.expanduser('~'), 'Application Data')
    return os.path.join(os.environ.get('APPDATA', default_path), 'DataHaven.NET')

def BaseDirMac():
    return os.path.join(os.path.expanduser('~'), '.datahaven')

def DefaultBaseDir():
    if dhnio.Windows():
        return BaseDirWindows()
    elif dhnio.Linux():
        return BaseDirLinux()
    elif dhnio.Mac():
        return BaseDirMac()
    return BaseDirDefault()

def BaseDir():
    global _BaseDirPath
    init()
    return _BaseDirPath

def BaseDirPathFileName():
    return os.path.join(dhnio.getExecutableDir(), 'basedir.txt')

_NeighbourBaseDirPath = ''
def NeighbourBaseDir():
    global _NeighbourBaseDirPath
    if _NeighbourBaseDirPath == '':
        _NeighbourBaseDirPath = str(os.path.abspath(os.path.join(
            dhnio.getExecutableDir(),
            '..',
            '.datahaven')))
    return _NeighbourBaseDirPath

def RestoreDir():
    return os.path.expanduser('~')
    #return BaseDir()

def WindowsBinDir():
    return os.path.join(BaseDir(), 'bin')

def RaidDir():
    return os.path.join(BaseDir(), "raiddir")

def MetaDataDir():
    return os.path.join(BaseDir(), "metadata")

def TempDir():
    return os.path.join(BaseDir(), "temp")

def IdentityCacheDir():
    # identitycache.py stores local copies of identities
    return os.path.join(BaseDir(), "identitycache")

def KeyFileName():
    return os.path.join(MetaDataDir(), "mykeyfile")

def SupplierIDsFilename():
    # IDs for places that store data for us
    return os.path.join(MetaDataDir(), "supplierids")

def CustomerIDsFilename():
    # IDs for places we store data for
    return os.path.join(MetaDataDir(), "customerids")

def CorrespondentIDsFilename():
    # people we get messages from
    return os.path.join(MetaDataDir(), "correspondentids")

def LocalIdentityFilename():
    return os.path.join(MetaDataDir(), "localidentity")

def LocalIPFilename():
    # file contains string like 192.168.100.33
    return os.path.join(MetaDataDir(), "localip")

def ExternalIPFilename():
    # file contains string like 207.42.133.2
    return os.path.join(MetaDataDir(), "externalip")

def DefaultTransportOrderFilename():
    return os.path.join(MetaDataDir(), "torder")

def UserNameFilename():
    # file contains something like "guesthouse"
    return os.path.join(MetaDataDir(), "username")

def UserConfigFilename():
    return os.path.join(MetaDataDir(),"userconfig")

def GUIOptionsFilename():
    return os.path.join(MetaDataDir(),"guioptions")

def UpdateSheduleFilename():
    return os.path.join(MetaDataDir(),"updateshedule")

def LocalPortFilename():
    return os.path.join(MetaDataDir(), 'localport')

def BackupInfoFileName():
    return "backup_info.xml"

def BackupInfoFileFullPath():
    return os.path.join(MetaDataDir(), BackupInfoFileName())

def InstalledDateFileName():
    return os.path.join(MetaDataDir(), 'installed')

#-------------------------------------------------------------------------------

def BackupsDBDir():
    return os.path.join(BaseDir(), 'backups')

def Q2QDir():
    return os.path.join(BaseDir(), 'q2qcerts')

def LogsDir():
    return os.path.join(BaseDir(), 'logs')

def SuppliersDir():
    return os.path.join(BaseDir(), 'suppliers')

def SupplierPath(idurl, filename=None):
    if filename is not None:
        return os.path.join(SuppliersDir(), nameurl.UrlFilename(idurl), filename)
    return os.path.join(SuppliersDir(), nameurl.UrlFilename(idurl))

def SupplierListFilesFilename(idurl):
    return os.path.join(SupplierPath(idurl), 'listfiles')

def RatingsDir():
    return os.path.join(BaseDir(), 'ratings')

def TransportLogFile():
    return (os.path.join(LogsDir(), 'transport.log'))

def DebugLogFile():
    return (os.path.join(LogsDir(), 'debug.log'))

def LocalTesterLogFilename():
    return os.path.join(LogsDir(), 'dhntester.log')

def MiniupnpcLogFilename():
    return os.path.join(LogsDir(), 'miniupnpc.log')

def MainLogFilename():
    return os.path.join(LogsDir(), 'dhn')

def TwistedLogFilename():
    return os.path.join(LogsDir(), 'twisted.log')

def UpdateLogFilename():
    return os.path.join(LogsDir(), 'dhnupdate.log')

def RunUpdateLogFilename():
    return os.path.join(LogsDir(), 'runupdate.log')

def SignerLogFilename():
    return os.path.join(LogsDir(), 'dhnsigner.log')

def RunupnpcLogFilename():
    return os.path.join(LogsDir(), 'runupnpc.log')

def FireHireLogFilename():
    return os.path.join(LogsDir(), 'fire_hire.log')

def CSpaceLogFilename():
    return os.path.join(LogsDir(), 'cspace.log')

_CentralDBDir = ''
def CentralDBDir():
    global _CentralDBDir
    if not _CentralDBDir:
        _CentralDBDir = os.path.expanduser('~')
        _CentralDBDir = os.path.join(_CentralDBDir, "dhndb")
    return _CentralDBDir

def SetCentralDBDir(dirpath):
    global _CentralDBDir
    _CentralDBDir = dirpath

def TransactionsDir():
    return os.path.join(CentralDBDir(), 'transactions')

def AccountsDir():
    return os.path.join(CentralDBDir(), 'accounts')

def CentralBandwidthDir():
    return os.path.join(CentralDBDir(), 'bandwidth')

def CentralWebStatsDir():
    return os.path.join(CentralDBDir(), 'webstats')

def CentralDailyReportsDir():
    return os.path.join(CentralDBDir(), 'dailyreports')

def CentralReceiptsDir():
    return os.path.join(CentralDBDir(), 'receipts')

def CentralDeadUsersReceiptsDir():
    return os.path.join(CentralDBDir(), 'receipts.deadusers')

def CentralRatingsDir():
    return os.path.join(CentralDBDir(), 'ratings')

def BandwidthInDir():
    return os.path.join(BaseDir(),"bandin")

def BandwidthOutDir():
    return os.path.join(BaseDir(),"bandout")

def CentralLogFile():
    return os.path.join(LogsDir(), "central.log")

def CentralBetaTestersFile():
    return '/home/veselin/datahaven/central/beta-test-users'

def RepoFile():
    return os.path.join(MetaDataDir(), 'repo')

def VersionFile():
    return os.path.join(MetaDataDir(), 'version')

def InfoFile():
    return os.path.join(MetaDataDir(), 'info')

def RevisionNumberFile():
    return os.path.join(dhnio.getExecutableDir(), 'revnum.txt')

def CustomersSpaceFile():
    return os.path.join(MetaDataDir(), 'space')

def BalanceFile():
    return os.path.join(MetaDataDir(), 'balance')

def CertificateFiles():
    return [    os.path.join(MetaDataDir(), 'dhn.cer'),
                os.path.join('.', 'dhn.cer'),
                os.path.join(dhnio.getExecutableDir() ,'dhn.cer'),]

def WindowsStarterFileName():
    return 'dhnstarter.exe'

def WindowsStarterFileURL(repo=DefaultRepo()):
    return UpdateLocationURL(repo) + 'windows/' + WindowsStarterFileName()

def getAutorunFilename():
    if dhnio.Windows() and dhnio.isFrozen():
        path = os.path.join(dhnio.getExecutableDir(), WindowsStarterFileName())
        if not os.path.isfile(path):
            path = dhnio.getExecutableFilename()
    else:
        path = dhnio.getExecutableFilename()
    return path

def getIconLaunchFilename():
    #return os.path.join(dhnio.getExecutableDir(), WindowsStarterFileName())
    return os.path.join(dhnio.getExecutableDir(), 'dhnmain.exe')

def getIconLinkFilename():
    return 'Data Haven .NET.lnk'

def IconFilename():
    return 'dhnicon.ico'

def IconsFolderPath():
    return os.path.join(dhnio.getExecutableDir(), 'icons')

#-------------------------------------------------------------------------------

def MerchantID():
    return 'AXA_DH_02666084001'
##    return 'AXA_DH_TESTKEY1'

def MerchantURL():
    return 'https://merchants.4csonline.com/TranSvcs/tp.aspx'
##    return 'https://merchants.4csonline.com/DevTranSvcs/tp.aspx'

#-------------------------------------------------------------------------------

def IdentityServerName():
    return "identity.datahaven.net"

def DefaultIdentityServer():
    return "identity.datahaven.net"

def DefaultQ2QServer():
    # return 'work.offshore.ai'
    return 'datahaven.net'

def MoneyServerName():
##    return 'localhost'
##    return '67.207.147.183' #central
##    return "central.net"
##    return "central.datahaven.net"
##    return "209.59.119.34" #offshore.ai
#    return 'datahaven.net'
    return "offshore.ai"

def UpdateServerName():
    return 'localhost'

def CentralStatsURL():
#    return 'http://work.offshore.ai/~veselin/' - #that's old location
    return 'http://identity.datahaven.net/statistics/'

#-------------------------------------------------------------------------------

def DefaultSSHPort():
    return 5022
# This should be same for all identity servers everywhere

def IdentityServerPort():
    return 7773

def IdentityWebPort():
    return 80

def MoneyServerPort():
    return 9898

def DefaultTCPPort():
    return 7771

def DefaultUDPPort():
    return 0

def UpdateServerPort():
    return 8301

def DefaultHTTPPort():
    return 9786

def DefaultWebLogPort():
    return 9999

def DefaultWebTrafficPort():
    return 9997

#-------------------------------------------------------------------------------

##def CustomersDataDir():
##    return uconfig("other.CustomerDataDirectory").strip()

##def MessagesDir():
##    return uconfig("other.MessagesDirectory").strip()
##
##def ReceiptsDir():
##    return uconfig("other.ReceiptsDirectory").strip()

def getCustomersFilesDir():
    return uconfig('folder.folder-customers').strip()

def getCustomerFilesDir(idurl):
    return os.path.join(getCustomersFilesDir(), nameurl.UrlFilename(idurl))

def getLocalBackupsDir():
    return uconfig('folder.folder-backups').strip()

def getRestoreDir():
    return uconfig('folder.folder-restore').strip()

def getMessagesDir():
    return uconfig('folder.folder-messages').strip()

def getReceiptsDir():
    return uconfig('folder.folder-receipts').strip()

def getTempDir():
    #if value is empty value - let's use default (system) temporary folder
    #return uconfig('folder.folder-temp').strip()
    return TempDir()

#------------------------------------------------------------------------------

def enableProxy(enable=None):
    if enable is None:
        return uconfig('network.network-proxy.network-proxy-enable').lower() == 'true'
    uconfig().set('network.network-proxy.network-proxy-enable', str(enable))
    uconfig().update()

def getProxyHost():
    return uconfig('network.network-proxy.network-proxy-host').strip()

def getProxyPort():
    return uconfig('network.network-proxy.network-proxy-port').strip()

def setProxySettings(d):
    if d.has_key('host'):
        uconfig().set('network.network-proxy.network-proxy-host', str(d.get('host','')))
    if d.has_key('port'):
        uconfig().set('network.network-proxy.network-proxy-port', str(d.get('port','')))
    if d.has_key('username'):
        uconfig().set('network.network-proxy.network-proxy-username', str(d.get('username','')))
    if d.has_key('password'):
        uconfig().set('network.network-proxy.network-proxy-password', str(d.get('password','')))
    if d.has_key('ssl'):
        uconfig().set('network.network-proxy.network-proxy-ssl', str(d.get('ssl','False')))
    uconfig().update()

def getProxySettingsDict():
    return {
         'host':        uconfig('network.network-proxy.network-proxy-host').strip(),
         'port':        uconfig('network.network-proxy.network-proxy-port').strip(),
         'username':    uconfig('network.network-proxy.network-proxy-username').strip(),
         'password':    uconfig('network.network-proxy.network-proxy-password').strip(),
         'ssl':         uconfig('network.network-proxy.network-proxy-ssl').strip(), }

def update_proxy_settings():
    import dhnnet
    dhnnet.init()
    if enableProxy():
        if getProxyHost() == '' or getProxyPort() == '':
            d = dhnnet.detect_proxy_settings()
            dhnnet.set_proxy_settings(d)
            setProxySettings(d)
            enableProxy(d.get('host', '') != '')
            dhnio.Dprint(2, 'settings.update_proxy_settings UPDATED!!!')
        else:
            dhnnet.set_proxy_settings(getProxySettingsDict())
        dhnio.Dprint(4, 'settings.update_proxy_settings')
        dhnio.Dprint(4, 'HOST:      ' + dhnnet.get_proxy_host())
        dhnio.Dprint(4, 'PORT:      ' + str(dhnnet.get_proxy_port()))
        dhnio.Dprint(4, 'USERNAME:  ' + dhnnet.get_proxy_username())
        dhnio.Dprint(4, 'PASSWORD:  ' + ('*' * len(dhnnet.get_proxy_password())))
        dhnio.Dprint(4, 'SSL:       ' + dhnnet.get_proxy_ssl())


#------------------------------------------------------------------------------

#def BandwidthLimit():          
#    global _BandwidthLimit
#    if _BandwidthLimit is None:
#        try:
#            #_BandwidthLimit = float(uconfig('other.BandwidthLimit'))
#            _BandwidthLimit = float(uconfig('network.network-bandwidth-limit'))
#        except:
#            _BandwidthLimit = 0 
#    return _BandwidthLimit

#def setBandwidthLimit(bandlimit):
#    global _BandwidthLimit
#    uconfig().set('network.network-bandwidth-limit', str(bandlimit))
#    uconfig().update()
#    _BandwidthLimit = float(bandlimit)

def getBandOutLimit(): # in kilo bytes per second
    try:
        return int(uconfig('network.network-send-limit'))
    except:
        return 0

def getBandInLimit(): # in kilo bytes per second
    try:
        return int(uconfig('network.network-receive-limit'))
    except:
        return 0

def FireInactiveSupplierIntervalHours():  # after a supplier hasn't been heard from in
    return uconfig('other.FireInactiveSupplierIntervalHours')

def ShowBarcode():  # do we want to show barcode stuff?  At the moment only for debuggers
    return uconfig('other.ShowBarcode')

def getTransportPort(proto):
    if proto == 'tcp':
        return getTCPPort()
    if proto == 'udp':
        return getUDPPort()
    if proto == 'ssh':
        return getSSHPort()
    if proto == 'http':
        return getHTTPPort()

def getTCPPort():
    return uconfig("transport.transport-tcp.transport-tcp-port")

def setTCPPort(port):
    uconfig().set("transport.transport-tcp.transport-tcp-port", str(port))
    uconfig().update()

def enableTCP(enable=None):
    if enable is None:
        return uconfig('transport.transport-tcp.transport-tcp-enable').lower() == 'true'
    uconfig().set('transport.transport-tcp.transport-tcp-enable', str(enable))
    uconfig().update()

def getUDPPort():
    return uconfig("transport.transport-udp.transport-udp-port")

def setUDPPort(port):
    uconfig().set("transport.transport-udp.transport-udp-port", str(port))
    uconfig().update()

def enableUDP(enable=None):
    if enable is None:
        return uconfig('transport.transport-udp.transport-udp-enable').lower() == 'true'
    uconfig().set('transport.transport-udp.transport-udp-enable', str(enable))
    uconfig().update()

def getSSHPort():
    return uconfig("transport.transport-ssh.transport-ssh-port")

def setSSHPort(port):
    uconfig().set("transport.transport-ssh.transport-ssh-port", str(port))
    uconfig().update()

def enableSSH(enable=None):
    if enable is None:
        return uconfig('transport.transport-ssh.transport-ssh-enable').lower() == 'true'
    uconfig().set('transport.transport-ssh.transport-ssh-enable', str(enable))
    uconfig().update()

def getQ2Qhost():
    return uconfig("transport.transport-q2q.transport-q2q-host")

def setQ2Qhost(host):
    uconfig().set("transport.transport-q2q.transport-q2q-host", host)
    uconfig().update()

def getQ2Qusername():
    return uconfig("transport.transport-q2q.transport-q2q-username")

def setQ2Qusername(username):
    uconfig().set("transport.transport-q2q.transport-q2q-username", username)
    uconfig().update()

def getQ2Qpassword():
    return uconfig("transport.transport-q2q.transport-q2q-password")

def setQ2Qpassword(password):
    uconfig().set("transport.transport-q2q.transport-q2q-password", password)
    uconfig().update()

def getQ2Quserathost():
    return getQ2Qusername()+'@'+getQ2Qhost()

def setQ2Quserathost(userAThost):
    username, host = userAThost.split('@')
    setQ2Qhost(host)
    setQ2Qusername(username)

def getCSpaceKeyID():
    return uconfig('transport.transport-cspace.transport-cspace-key-id')

def setCSpaceKeyID(key_id):
    uconfig().set('transport.transport-cspace.transport-cspace-key-id', key_id)
    uconfig().update()

def enableQ2Q(enable=None):
    if enable is None:
        return uconfig('transport.transport-q2q.transport-q2q-enable').lower() == 'true'
    uconfig().set('transport.transport-q2q.transport-q2q-enable', str(enable))
    uconfig().update()

def getHTTPPort():
    return uconfig('transport.transport-http.transport-http-server-port')

def setHTTPPort(port):
    uconfig().set("transport.transport-http.transport-http-port", str(port))
    uconfig().update()

def getHTTPDelay():
    return dhnmath.toInt(uconfig('transport.transport-http.transport-http-ping-timeout'), DefaultHTTPDelay())

def enableHTTP(enable=None):
    if enable is None:
        return uconfig('transport.transport-http.transport-http-enable').lower() == 'true'
    uconfig().set('transport.transport-http.transport-http-enable', str(enable))
    uconfig().update()

def enableHTTPServer(enable=None):
    if enable is None:
        return uconfig('transport.transport-http.transport-http-server-enable').lower() == 'true'
    uconfig().set('transport.transport-http.transport-http-server-enable', str(enable))
    uconfig().update()

def enableCSpace(enable=None):
    if enable is None:
        return uconfig('transport.transport-cspace.transport-cspace-enable').lower() == 'true'
    uconfig().set('transport.transport-cspace.transport-cspace-enable', str(enable))
    uconfig().update()

def SendTimeOutEmail():
    return uconfig("other.emailSendTimeout")

def DefaultReceiveTimeOutEmail():
    return 120                # seconds receive timeout for email

def DefaultHTTPDelay():
    return 5

def ReceiveTimeOutEmail():
    return uconfig("other.emailReceiveTimeout")

def EmailPollingTime():
    return 30

def getEmailAddress():
    return uconfig('transport.transport-email.transport-email-address')

def getPOPHost():
    return uconfig('transport.transport-email.transport-email-pop-host')

def getPOPPort():
    return uconfig('transport.transport-email.transport-email-pop-port')

def getPOPUser():
    return uconfig('transport.transport-email.transport-email-pop-username')

def getPOPPass():
    return uconfig('transport.transport-email.transport-email-pop-password')

def getPOPSSL():
    return uconfig('transport.transport-email.transport-email-pop-ssl')

def getSMTPHost():
    return uconfig('transport.transport-email.transport-email-smtp-host')

def getSMTPPort():
    return uconfig('transport.transport-email.transport-email-smtp-port')

def getSMTPUser():
    return uconfig('transport.transport-email.transport-email-smtp-username')

def getSMTPPass():
    return uconfig('transport.transport-email.transport-email-smtp-password')

def getSMTPSSL():
    return uconfig('transport.transport-email.transport-email-smtp-ssl')

def getSMTPNeedLogin():
    return uconfig('transport.transport-email.transport-email-smtp-need-login').lower() == 'true'

def enableEMAIL(enable=None):
    if enable is None:
        return uconfig('transport.transport-email.transport-email-enable').lower() == 'true'
    uconfig().set('transport.transport-email.transport-email-enable', str(enable))
    uconfig().update()

def enableSKYPE(enable=None):
    if enable is None:
        return uconfig('transport.transport-skype.transport-skype-enable').lower() == 'true'
    uconfig().set('transport.transport-skype.transport-skype-enable', str(enable))
    uconfig().update()

def enableTransport(proto, enable=None):
    key = 'transport.transport-%s.transport-%s-enable' % (proto, proto)
    if uconfig(key) is None:
        return False
    if enable is None:
        return uconfig(key).lower() == 'true'
    uconfig().set(key, str(enable))
    uconfig().update()

def transportIsEnabled(proto):
    if proto == 'http':
        return enableHTTPServer() or enableHTTP()
    return enableTransport(proto)

def transportIsInstalled(proto):
    if proto == 'q2q':
        return getQ2Qusername().strip() != '' and getQ2Qhost().strip() != ''
    if proto == 'email':
        return getEmailAddress().strip() != ''
    if proto == 'cspace':
        return getCSpaceKeyID().strip() != ''
    return True

#debug level info
def getDebugLevelStr(): # this is just for checking if it is set, the int() would throw an error
#    return uconfig("other.DebugLevel")
    return uconfig("logs.debug-level")

def getDebugLevel():
    try:
#        res = int(uconfig("other.DebugLevel"))
        res = int(getDebugLevelStr())
    except:
        res = dhnio.DebugLevel
    return res

def setDebugLevel(level):
#    uconfig().set("other.DebugLevel", str(level))
    uconfig().set("logs.debug-level", str(level))
    uconfig().update()

def enableWebStream(enable=None):
    if enable is None:
        return uconfig('logs.stream-enable').lower() == 'true'
    uconfig().set('logs.stream-enable', str(enable))
    uconfig().update()

def enableWebTraffic(enable=None):
    if enable is None:
        return uconfig('logs.traffic-enable').lower() == 'true'
    uconfig().set('logs.traffic-enable', str(enable))
    uconfig().update()

def getWebStreamPort():
    try:
        return int(uconfig('logs.stream-port'))
    except:
        return DefaultWebLogPort()

def getWebTrafficPort():
    try:
        return int(uconfig('logs.traffic-port'))
    except:
        return DefaultWebTrafficPort()
    
def enableMemoryProfile(enable=None):
    if enable is None:
        return uconfig('logs.memprofile-enable').lower() == 'true'
    uconfig().set('logs.memprofile-enable', str(enable))
    uconfig().update()

def getDozerStr():
    return uconfig('other.Dozer')

def getDoBackupMonitor():
    return uconfig('other.DoBackupMonitor')

def getECC():
    snum = getCentralNumSuppliers()
    if snum < 0:
        return DefaultEccMapName()
    ecc = eccmap.GetEccMapName(snum)
    if isValidECC(ecc):
        return ecc
    else:
        return DefaultEccMapName()

def getECCSuppliersNumbers():
    return eccmap.SuppliersNumbers()

def getCentralNumSuppliers():
    try:
        return int(uconfig('central-settings.desired-suppliers'))
    except:
        return -1

def getCentralMegabytesNeeded():
    return uconfig('central-settings.needed-megabytes')

def getCentralMegabytesDonated():
    return uconfig('central-settings.shared-megabytes')

def getEmergencyEmail():
    return uconfig('emergency.emergency-email')

def getEmergencyPhone():
    return uconfig('emergency.emergency-phone')

def getEmergencyFax():
    return uconfig('emergency.emergency-fax')

def getEmergencyOther():
    return uconfig('emergency.emergency-text')

def getEmergency(method):
    if method not in getEmergencyMethods():
        return ''
    return uconfig('emergency.emergency-' + method)

def getEmergencyFirstMethod():
    return uconfig('emergency.emergency-first')

def getEmergencySecondMethod():
    return uconfig('emergency.emergency-second')

def getEmergencyMethods():
    return (
        'email',
        'phone',
        'fax',
        'other',)

def getUpdatesMode():
    return uconfig('updates.updates-mode')

def getUpdatesModeValues():
    return  (
        'install automatically',
#        'ask before install',
        'only notify',
        'turn off updates', )

def getUpdatesSheduleData():
    return uconfig('updates.updates-shedule')

def setUpdatesSheduleData(raw_shedule):
    uconfig().set('updates.updates-shedule', raw_shedule)
    uconfig().update()

def getGeneralBackupsToKeep():
    try:
        return int(uconfig('general.general-backups'))
    except:
        return 2

def getGeneralLocalBackups():
    return uconfig('general.general-local-backups-enable').lower() == 'true'

def getGeneralWaitSuppliers():
    return uconfig('general.general-wait-suppliers-enable').lower() == 'true'

def getGeneralAutorun():
    return uconfig('general.general-autorun').lower() == 'true'

def getGeneralDisplayMode():
    return uconfig('general.general-display-mode')

def getGeneralDisplayModeValues():
    return ('iconify window', 'normal window', 'maximized window',)

##def getGeneralShowProgress():
##    return uconfig('general.general-show-progress').lower() == 'true'

def getGeneralDesktopShortcut():
    return uconfig('general.general-desktop-shortcut').lower() == 'true'

def getGeneralStartMenuShortcut():
    return uconfig('general.general-start-menu-shortcut').lower() == 'true'

def getSystemUserDataPathList():
    return uconfig().get_childs('system.system-user-data-paths')

def getGuiFontSize():
    try:
        return int(uconfig('gui.gui-font-size'))
    except:
        return 12

def enableUPNP(enable=None):
    if enable is None:
        return uconfig('other.upnp-enabled').lower() == 'true'
    uconfig().set('other.upnp-enabled', str(enable))
    uconfig().update()

def getUPNPatStartup():
    return uconfig('other.upnp-at-startup').lower() == 'true'

def setUPNPatStartup(enable):
    uconfig().set('other.upnp-at-startup', str(enable))
    uconfig().update()

#-------------------------------------------------------------------------------

def isValidECC(ecc):
##    if ecc in validECCs():
    if ecc in eccmap.EccMapNames():
        return True
    else:
        return False

#-------------------------------------------------------------------------------

def RenameBaseDir(newdir):
    global _BaseDirPath
    olddir = _BaseDirPath
    try:
#        os.renames(_BaseDirPath, newdir) # - this not fit for us.
        import shutil
        shutil.copytree(olddir, newdir)
    except:
        dhnio.DprintException()
        return False
    _BaseDirPath = newdir
    dhnio.Dprint(2, 'settings.RenameBaseDir  directory was copied,  BaseDir='+BaseDir())
    pathfilename = BaseDirPathFileName()
    dhnio.WriteFile(pathfilename, _BaseDirPath)
    dhnio.Dprint(4, 'settings.RenameBaseDir  BaseDir path was saved to ' + pathfilename)
    logfilename = dhnio.LogFileName
    dhnio.CloseLogFile()
    try:
        dhnio.rmdir_recursive(olddir, True)
        dhnio.Dprint(4, 'settings.RenameBaseDir  old directory was removed: ' + olddir)
    except:
        dhnio.DprintException()
    dhnio.OpenLogFile(logfilename, True)
    return True

def _initBaseDir(base_dir=None):
    global _BaseDirPath
    
    # if we already know the place - we are done
    if base_dir is not None:
        _BaseDirPath = base_dir
        return

    # if we have a file 'basedir.txt' in current folder - take the place from there
    if os.path.isfile(BaseDirPathFileName()):
        path = dhnio.ReadBinaryFile(BaseDirPathFileName())
        if os.path.isdir(path):
            _BaseDirPath = path
            return

    # get the default place for thet machine
    default_path = DefaultBaseDir()

    # we can use folder ".datahaven" placed on the same level with binary folder:
    # /..
    #   /.datahaven - data files
    #   /datahaven  - binary files
    path1 = str(os.path.abspath(os.path.join(dhnio.getExecutableDir(), '..', '.datahaven')))
    # and default path will have lower priority
    path2 = default_path
    
    # if default path exists - use it
    if os.path.isdir(path2):
        _BaseDirPath = path2
    # but .datahaven on same level will have bigger priority
    if os.path.isdir(path1):
        _BaseDirPath = path1

    # ?? it think I should comment those 4 lines
    if dhnio.Linux():
        path2 = BaseDirLinux()
    elif dhnio.Mac():
        path2 = BaseDirMac()

    # if we did not found "metadata" subfolder - use default path, new copy of DHN
    if not os.path.isdir(MetaDataDir()):
        _BaseDirPath = path2
        return
    
    # if we did not found our key - use default path, new copy of DHN
    if not os.access(KeyFileName(), os.R_OK):
        _BaseDirPath = path2
        return
    
    # if we did not found our identity - use default path, new copy of DHN
    if not os.access(LocalIdentityFilename(), os.R_OK):
        _BaseDirPath = path2
        return

    # if we did not found our config - use default path, new copy of DHN
    if not os.access(UserConfigFilename(), os.R_OK):
        _BaseDirPath = path2
        return

    # if we did not found our suppliers - use default path, new copy of DHN
    if not os.access(SupplierIDsFilename(), os.R_OK):
        _BaseDirPath = path2
        return

    # if we did not found our customers - use default path, new copy of DHN
    if not os.access(CustomerIDsFilename(), os.R_OK):
        _BaseDirPath = path2
        return


def _checkSettings():
    if getCentralNumSuppliers() < 0:
        uconfig().set("central-settings.desired-suppliers", str(DefaultDesiredSuppliers()))

    if getCentralMegabytesDonated() == '':
        uconfig().set("central-settings.shared-megabytes", str(DefaultDonatedMb())+' Mb')
    donatedV, donatedS = diskspace.SplitString(getCentralMegabytesDonated())
    if not donatedS:
        uconfig().set("central-settings.shared-megabytes", str(getCentralMegabytesDonated())+' Mb')

    if getCentralMegabytesNeeded() == '':
        uconfig().set("central-settings.needed-megabytes", str(DefaultNeededMb())+' Mb')
    neededV, neededS = diskspace.SplitString(getCentralMegabytesNeeded())
    if not neededS:
        uconfig().set("central-settings.needed-megabytes", str(getCentralMegabytesNeeded())+' Mb')

    if getDebugLevelStr() == "":
        uconfig().set("logs.debug-level", str(defaultDebugLevel()))

    if getCustomersFilesDir() == '':
        uconfig().set('folder.folder-customers', os.path.join(BaseDir(), "customers"))
        
    if getLocalBackupsDir() == '':
        if os.path.isdir(os.path.join(TempDir(), "data-par")):
            uconfig().set('folder.folder-backups', os.path.join(TempDir(), "data-par"))
        else:
            uconfig().set('folder.folder-backups', os.path.join(BaseDir(), "backups"))

    if getRestoreDir() == '':
        uconfig().set('folder.folder-restore', RestoreDir())

    if getMessagesDir() == '':
        uconfig().set('folder.folder-messages', os.path.join(BaseDir(), 'messages'))

    if getReceiptsDir() == '':
        uconfig().set('folder.folder-receipts', os.path.join(BaseDir(), 'receipts'))

#    if getBandOutLimit() == "":
#        #uconfig().set("other.BandwidthLimit", str(DefaultBandwidthLimit()))
#        uconfig().set("network.network-send-limit", str(DefaultBandwidthLimit()))
#
#    if getBandInLimit() == "":
#        #uconfig().set("other.BandwidthLimit", str(DefaultBandwidthLimit()))
#        uconfig().set("network.network-receive-limit", str(DefaultBandwidthLimit()))

    if SendTimeOutEmail() == "":
        uconfig().set("other.emailSendTimeout", str(DefaultSendTimeOutEmail()))

    if ReceiveTimeOutEmail() == "":
        uconfig().set("other.emailReceiveTimeout", str(DefaultReceiveTimeOutEmail()))

    if getTCPPort() == "":
        uconfig().set("transport.transport-tcp.transport-tcp-port", str(DefaultTCPPort()))

    if getUDPPort() == "":
        uconfig().set("transport.transport-udp.transport-udp-port", str(DefaultUDPPort()))

    if getSSHPort() == "":
        uconfig().set("transport.transport-ssh.transport-ssh-port", str(DefaultSSHPort()))

    if getHTTPPort() == "":
        uconfig().set("transport.transport-http.transport-http-port", str(DefaultHTTPPort()))

    if getUpdatesMode().strip() not in getUpdatesModeValues():
        uconfig().set('updates.updates-mode', getUpdatesModeValues()[0])

    if getGeneralDisplayMode().strip() not in getGeneralDisplayModeValues():
        uconfig().set('general.general-display-mode', getGeneralDisplayModeValues()[0])

    if getEmergencyFirstMethod() not in getEmergencyMethods():
        uconfig().set('emergency.emergency-first', getEmergencyMethods()[0])

    if getEmergencySecondMethod() not in getEmergencyMethods():
        uconfig().set('emergency.emergency-second', getEmergencyMethods()[1])

    if getEmergencyFirstMethod() == getEmergencySecondMethod():
        methods = list(getEmergencyMethods())
        methods.remove(getEmergencyFirstMethod())
        uconfig().set('emergency.emergency-second', methods[0])

    uconfig().update()


def _checkStaticDirectories():
    # check that the base directory exists
    if not os.path.isdir(BaseDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + BaseDir())
        dhnio._dirs_make(BaseDir())
        if dhnio.Windows():
            _initBaseDir()

    # check that the metadata directory exists
    if not os.path.exists(MetaDataDir()): 
        dhnio.Dprint(8, 'settings.init want to create folder: ' + MetaDataDir())
        #dhnio._dirs_make(MetaDataDir())
        os.makedirs(MetaDataDir())

#        # but if we see 'metadata'
#        if os.path.exists("metadata"):
#            os.rename("metadata", MetaDataDir())
#        else:
#            dhnio.Dprint(8, 'settings.init want to create folder: ' + MetaDataDir())
#            dhnio._dir_make(MetaDataDir())

#    if not os.path.exists(Q2QDir()):
#        if os.path.exists("q2qcerts"):
#            os.rename("q2qcerts", Q2QDir())
#        else:
#            dhnio.Dprint(8, 'settings.init want to create folder: ' + Q2QDir())
#            dhnio._dir_make(Q2QDir())

    # check that the raiddir directory exists
#    if not os.path.exists(RaidDir()):
#        if os.path.exists("raiddir"):
#            os.rename("raiddir", RaidDir())
#        else:
#            dhnio.Dprint(8, 'settings.init want to create folder: ' + RaidDir())
#            dhnio._dir_make(RaidDir())

    if not os.path.exists(BandwidthInDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + BandwidthInDir())
        os.makedirs(BandwidthInDir())
    if os.path.exists(os.path.join(BaseDir(),"bandwidthin")):
        dhnio.Dprint(2, 'settings.init want to delete folder: ' + os.path.join(BaseDir(),"bandwidthin"))
        dhnio.rmdir_recursive(os.path.join(BaseDir(),"bandwidthin"))

    if not os.path.exists(BandwidthOutDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + BandwidthOutDir())
        os.makedirs(BandwidthOutDir())
    if os.path.exists(os.path.join(BaseDir(),"bandwidthout")):
        dhnio.Dprint(2, 'settings.init want to delete folder: ' + os.path.join(BaseDir(),"bandwidthout"))
        dhnio.rmdir_recursive(os.path.join(BaseDir(),"bandwidthout"))

    if not os.path.exists(LogsDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + LogsDir())
        os.makedirs(LogsDir())

    if not os.path.exists(IdentityCacheDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + IdentityCacheDir())
        os.makedirs(IdentityCacheDir())

    if not os.path.exists(SuppliersDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + SuppliersDir())
        os.makedirs(SuppliersDir())

    if not os.path.exists(RatingsDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + RatingsDir())
        os.makedirs(RatingsDir())

    if not os.path.exists(TempDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + TempDir())
        os.makedirs(TempDir())

    # move files from previous location
    # we should check if same files not exist in current location
    # TODO - remove this in future versions
    # just need to update users now
#    if dhnio.Windows():
#        oldBaseDir = os.path.join(os.path.expanduser('~'), 'Application Data', 'DataHavenNet')
#        if os.path.isdir(oldBaseDir):
#            if not os.path.exists(KeyFileName()) and os.path.isfile(os.path.join(oldBaseDir, 'metadata', 'mykeyfile')):
#                os.rename(os.path.join(oldBaseDir, 'metadata', 'mykeyfile'), KeyFileName())
#            if not os.path.exists(LocalIdentityFilename()) and os.path.isfile(os.path.join(oldBaseDir, 'metadata', 'localidentity')):
#                os.rename(os.path.join(oldBaseDir, 'metadata', 'localidentity'), LocalIdentityFilename())


def _checkCustomDirectories():
    # check that the customer directory exists
    if not os.path.exists(getCustomersFilesDir()):
        if os.path.exists("customers"):
            os.renames("customers", getCustomersFilesDir())
        else:
            dhnio.Dprint(8, 'settings.init want to create folder: ' + getCustomersFilesDir())
            try:
                os.makedirs(getCustomersFilesDir())
            except:
                dhnio.Dprint(1, 'settings.init ERROR can not create folder: ' + getCustomersFilesDir())

    if not os.path.exists(getMessagesDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + getMessagesDir())
        os.makedirs(getMessagesDir())

    if not os.path.exists(getReceiptsDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + getReceiptsDir())
        os.makedirs(getReceiptsDir())
        
    if not os.path.exists(getLocalBackupsDir()):
        dhnio.Dprint(8, 'settings.init want to create folder: ' + getLocalBackupsDir())
        os.makedirs(getLocalBackupsDir())
    

#-------------------------------------------------------------------------------

def _init(base_dir=None):
    dhnio.Dprint(4, 'settings._init')
    dhnio.InstallLocale()
    _initBaseDir(base_dir)
    dhnio.Dprint(2, 'settings._init data location: ' + BaseDir())
    _checkStaticDirectories()
    uconfig()
    _checkSettings()
    _checkCustomDirectories()

#-------------------------------------------------------------------------------

if __name__ == '__main__':
    init()


    #debug
##    import pprint
##    pprint.pprint(uconfig().data)
##    print getSystemUserDataPathList()
##    for k,v in uconfig().data.items(): print k, v
##    print getSystemUserDataPathList()
##    enableTCP(False)
##    print type(enableTCP()), enableTCP()




